(* bzpdata.ml     Represent Bezier paths as data
 *
 * Copyright (c) 2011 Psellos   http://psellos.com
 * Licensed under the MIT license:
 *     http://www.opensource.org/licenses/mit-license.php
 *)

type bzp_elem =
| BZBegin
| BZMove of Cocoa.point
| BZLine of Cocoa.point
| BZCurve of Cocoa.point * Cocoa.point * Cocoa.point (* Cp1 Cp2 Endp *)
| BZClose

type bzpath = bzp_elem list


let bzp_pmap pfun bzpath =
    let efun = function
    | BZMove p -> BZMove (pfun p)
    | BZLine p -> BZLine (pfun p)
    | BZCurve (c1, c2, p) -> BZCurve (pfun c1, pfun c2, pfun p)
    | (BZBegin | BZClose) as e -> e
    in
        List.map efun bzpath


let bzp_iter efun bzpath =
    List.iter efun bzpath


let bbox bzpath =
    (* Return bounding box for the path.  We assume the path starts with
     * BZMove (so initial position is immaterial) and that its curves
     * aren't too erratic (they stay inside the rectangle bounded by
     * start and end points).  These are true for the paths we work with
     * here.
     *)
    let bbmax (xmin, xmax, ymin, ymax  as bb) elem =
        match elem with
        | BZMove (x, y)
        | BZLine (x, y)
        | BZCurve (_, _, (x, y)) ->
            (min x xmin, max x xmax, min y ymin, max y ymax)
        | _ -> bb
    in let (xmin, xmax, ymin, ymax) =
        List.fold_left bbmax (1_000_000.0, 0.0, 1_000_000.0, 0.0) bzpath
    in
        (xmin, ymin, xmax -. xmin, ymax -. ymin)


let bzp_scale bzpath (x, y, w, h) =
    let (bx, by, bw, bh) = bbox bzpath
    in let s, tx, ty =
        if (w /. bw) < (h /. bh) then
            let s = w /. bw
            in let tx = x -. s *. bx
            in let ty = y -. s *. by -. (s *. bh -. h) /. 2.0
            in
                s, tx, ty
        else
            let s = h /. bh
            in let tx = x -. s *. bx -. (s *. bw -. w) /. 2.0
            in let ty = y +. h -. s *. (by +. bh)
            in
                s, tx, ty
    in let fx x = s *. x +. tx
    in let fy y = s *. (by +. by +. bh -. y) +. ty
    in let fp (x, y) = (fx x, fy y)
    in
        bzp_pmap fp bzpath


(* Graphically interesting figure.
 *)
let figure1 = [
    BZBegin;
    BZMove  ( 49.922,  38.438);
    BZCurve (( 55.234,  38.438), ( 59.268,  39.336), ( 62.021,  41.133));
    BZCurve (( 64.775,  42.930), ( 66.611,  45.117), ( 67.529,  47.695));
    BZCurve (( 68.447,  50.273), ( 69.043,  54.570), ( 69.316,  60.586));
    BZCurve (( 69.590,  67.266), ( 71.084,  72.236), ( 73.799,  75.498));
    BZCurve (( 76.514,  78.760), ( 80.977,  80.391), ( 87.188,  80.391));
    BZLine  ( 87.188,  78.164);
    BZCurve (( 84.961,  78.008), ( 83.447,  77.139), ( 82.646,  75.557));
    BZCurve (( 81.846,  73.975), ( 81.445,  70.625), ( 81.445,  65.508));
    BZCurve (( 81.445,  58.008), ( 80.654,  52.217), ( 79.072,  48.135));
    BZCurve (( 77.490,  44.053), ( 74.316,  40.742), ( 69.551,  38.203));
    BZCurve (( 64.785,  35.664), ( 58.242,  34.297), ( 49.922,  34.102));
    BZLine  ( 49.922,  15.527);
    BZCurve (( 49.922,  10.957), ( 50.186,   7.998), ( 50.713,   6.650));
    BZCurve (( 51.240,   5.303), ( 52.354,   4.219), ( 54.053,   3.398));
    BZCurve (( 55.752,   2.578), ( 58.379,   2.168), ( 61.934,   2.168));
    BZLine  ( 61.934,   0.000);
    BZLine  ( 26.660,   0.000);
    BZLine  ( 26.660,   2.168);
    BZCurve (( 30.137,   2.207), ( 32.725,   2.588), ( 34.424,   3.311));
    BZCurve (( 36.123,   4.033), ( 37.256,   5.088), ( 37.822,   6.475));
    BZCurve (( 38.389,   7.861), ( 38.672,  10.879), ( 38.672,  15.527));
    BZLine  ( 38.672,  34.102);
    BZCurve (( 30.273,  34.297), ( 23.750,  35.645), ( 19.102,  38.145));
    BZCurve (( 14.453,  40.645), ( 11.299,  43.916), (  9.639,  47.959));
    BZCurve ((  7.979,  52.002), (  7.148,  57.871), (  7.148,  65.566));
    BZCurve ((  7.148,  70.488), (  6.777,  73.760), (  6.035,  75.381));
    BZCurve ((  5.293,  77.002), (  3.750,  77.930), (  1.406,  78.164));
    BZLine  (  1.406,  80.391);
    BZCurve ((  5.352,  80.312), (  8.535,  79.688), ( 10.957,  78.516));
    BZCurve (( 13.379,  77.344), ( 15.322,  75.273), ( 16.787,  72.305));
    BZCurve (( 18.252,  69.336), ( 19.082,  65.430), ( 19.277,  60.586));
    BZCurve (( 19.551,  54.609), ( 20.127,  50.332), ( 21.006,  47.754));
    BZCurve (( 21.885,  45.176), ( 23.730,  42.979), ( 26.543,  41.162));
    BZCurve (( 29.355,  39.346), ( 33.398,  38.438), ( 38.672,  38.438));
    BZLine  ( 38.672,  63.984);
    BZCurve (( 38.672,  68.555), ( 38.408,  71.514), ( 37.881,  72.861));
    BZCurve (( 37.354,  74.209), ( 36.250,  75.273), ( 34.570,  76.055));
    BZCurve (( 32.891,  76.836), ( 30.254,  77.246), ( 26.660,  77.285));
    BZLine  ( 26.660,  79.453);
    BZLine  ( 61.934,  79.453);
    BZLine  ( 61.934,  77.285);
    BZCurve (( 58.340,  77.246), ( 55.703,  76.846), ( 54.023,  76.084));
    BZCurve (( 52.344,  75.322), ( 51.240,  74.258), ( 50.713,  72.891));
    BZCurve (( 50.186,  71.523), ( 49.922,  68.555), ( 49.922,  63.984));
    BZClose;
    (* BZMove  ( 88.535,   0.000); *)
]
